@@ -1,14 +1,14 @@
-# Tie::Hash::Expire::Dumb
 package Cache::Simple::TimedExpiry;
 use strict;
 
 use vars qw/$VERSION/;
 
-$VERSION = '0.20';
+$VERSION = '0.24';
 
 # 0 - expiration delay
 # 1 - hash
 # 2 - expiration queue
+# 3 - last expiration
 
 =head2 new
 
@@ -18,7 +18,7 @@ Set up a new cache object
 
 
 sub new {
-  bless [2,{},[]], "Cache::Simple::TimedExpiry";
+  bless [2,{},[],0], "Cache::Simple::TimedExpiry";
 }
 
 
@@ -46,7 +46,7 @@ Return true if the cache has an entry with the key KEY
 sub has_key ($$) { # exists
   my ($self, $key) = @_;
   
-  return 1 if  $key && exists $self->[1]->{$key};
+  return 1 if defined $key && exists $self->[1]->{$key};
   return 0;
 }
 
@@ -55,12 +55,18 @@ sub has_key ($$) { # exists
 Return the cache entry with key KEY.
 Returns undef if there is no such entry
 
+(Can also be called as L<get>)
+
 =cut
 
+*get = \&fetch;
+
 sub fetch ($$) {
   my ($self,$key) = @_;
 
-    $self->expire();
+  my $time = time;
+  # Only expire 
+  $self->expire($time) if ($time > $self->[3]);
     unless ( $self->has_key($key)) {
           return undef;
      }
@@ -74,22 +80,31 @@ sub fetch ($$) {
 Store VALUE in the cache with accessor KEY.  Expire it from the cache 
 at or after EXPIRYTIME.
 
+(Can also be called as L<set>)
+
 =cut
 
-sub set ($$$) {
+*set = \&store;
+
+sub store ($$$) {
   my ($self,$key,$value) = @_;
+  my $time = time;
+  # Only expire 
+  $self->expire($time) if ($time > $self->[3]);
 
-  $self->expire();
-
+  return undef unless ($key);
   $self->[1]->{$key} = $value;
 
     push @{$self->[2]}, [ time, $key ];
 }
 
-sub expire ($) {
-  my ($self) = @_;
+sub expire ($$) {
+  my $self = shift;
+  my $time = shift;
+    
+  $self->[3] = $time;
 
-  my $oldest_nonexpired_entry = (time - $self->[0]);
+  my $oldest_nonexpired_entry = ($time - $self->[0]);
  
 
   return unless defined $self->[2]->[0]; # do we have an element in the array?
@@ -99,16 +114,18 @@ sub expire ($) {
 
   while ( @{$self->[2]} && $self->[2]->[0]->[0] <$oldest_nonexpired_entry ) {
     my $key =  $self->[2]->[0]->[1];
-    delete $self->[1]->{ $key } if exists $self->[1]->{$key};
+    delete $self->[1]->{ $key };
     shift @{$self->[2]};
   }
 
 }
 
 sub elements ($) { # keys
-  my ($self) = @_;
+  my $self = shift;
+  my $time = time;
+  # Only expire 
+  $self->expire($time) if ($time > $self->[3]);
 
-  $self->expire();
   return keys %{$self->[1]};
 
 }
@@ -146,6 +163,13 @@ $h->dump;
 
 print time;
 
+=head1 AUTHOR
+
+Jesse Vincent <jesse@bestpractical.com>
+Some of the heavy lifting was designed by Robert Spier <rspier@pobox.com>
+
+Copyright 2004 Jesse Vincent <jesse@bestpractical.com>
+
 =cut
 
 1;
